
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <unistd.h>
#include <errno.h>
#include <sys/mman.h>

#include "vx/vx.h"
#include "vx/chunk.h"
#include "env/env.h"


struct vxchunk *vxchunk_fromfd(int fd, off_t ofs, int closefd)
{
	struct vxchunk *ch = calloc(1, sizeof(vxchunk));
	if (ch == NULL) {
		errno = ENOMEM;
		return NULL;
	}

	ch->obj.type = VXOBJ_CHUNK;
	ch->fd = fd;
	ch->fofs = ofs;
	ch->closefd = closefd;

	// Initialize page permissions to default values:
	// read and write, but no execute.
	for (int i = 0; i < VXCHUNKSIZE/VXPAGESIZE; i++)
		ch->perm[i] = VXPERM_READ | VXPERM_WRITE;

	return ch;
}

struct vxchunk *vxchunk_new(const char *data, int len)
{
	assert(data == 0);	// XXX

	// Create a temporary file to represent the chunk
	char tmpfn[] = "/tmp/vxXXXXXX";
	int fd = mkstemp(tmpfn);
	if (fd < 0)
		return NULL;

	// Immediately remove it now that we have it open
	(void)remove(tmpfn);

	// Set the file's length to hold exactly one chunk worth of data.
	if (ftruncate(fd, VXCHUNKSIZE) < 0)
		goto err1;

	// Create the chunk
	struct vxchunk *ch = vxchunk_fromfd(fd, 0, 1);
	if (ch == NULL)
		goto err1;

	return ch;

err1:
	close(fd);
	return NULL;
}

vxchunk *vxchunk_alias(vxchunk *ch, int perm)
{
	// XXX perm

	// Duplicate the file descriptor if appropriate
	int fd = ch->fd;
	if (ch->closefd) {
		fd = dup(fd);
		if (fd < 0)
			return NULL;
	}

	// Create the new chunk
	vxchunk *nc = vxchunk_fromfd(fd, ch->fofs, ch->closefd);
	if (nc == NULL) {
		if (ch->closefd)
			close(fd);
		return NULL;
	}

	return nc;
}

void *vxchunk_map(vxchunk *ch, int perm)
{
	return mmap(NULL, VXCHUNKSIZE, perm, MAP_SHARED, ch->fd, ch->fofs);
}

void vxchunk_unmap(void *p)
{
	munmap(p, VXCHUNKSIZE);
}

void vxchunk_free(vxchunk *ch)
{
	if (ch->closefd)
		close(ch->fd);
	free(ch);
}

