//
// Main private header file for portable vx32 interpreter.
//
#ifndef ENV_EMU_H
#define ENV_EMU_H

#include <inttypes.h>
#include <setjmp.h>

#include <vx/proc.h>			// for vxpcallhandler


// Emulation state for vx32 environment.
struct vxemu {

	// Back pointer to vxproc
	struct vxproc	*proc;

	// vx32 virtual CPU state
	uint32_t	reg[8];
	uint32_t	eip;
	uint32_t	eflags;

	// vx32 memory (mirrors fields in vxproc struct)
	uint8_t		*mem;
	uint32_t	memsize;

	// Virtual trap handling state
	uint32_t	trapno;
	uint32_t	traperr;
	uint32_t	trapva;
	jmp_buf		*trapenv;

	// General emulation state
	vxpcallhandler	*pcallhandler;	// Registered parent-call handler
	void		*pcalldata;	// Opaque data assoc'd with handler
	size_t		maxmsglen;	// Maximum data to recv on pcalls	
	uint32_t	emuflags;	// Emulation flags (see below)
	uint32_t	jmpinfo;	// for jump insn backpatching

	// Ephemeral instruction interpretation state
	uint8_t		*ininst;	// Start of current instruction
	uint8_t		stmode;		// Status flag set mode
	uint32_t	starg1;		// Status flag arguments
	uint32_t	starg2;		// Status flag arguments

	// Max # of instructions to execute before returning
	uint32_t	insncount;
};
typedef struct vxemu vxemu;

// Emulation flags (emuflags)
#define EMU_FP		0x01	// Enable SSE2 scalar floating-point
#define EMU_VFP		0x02	// Enable vector floating-point
#define EMU_VINT	0x04	// Enable vector integer arithmetic


// Status flag set mode
#define STM_MODE	0x1f	// Mode field
#define STM_SIZE	0xc0	// Operand size field

#define STM_EFLAGS	0x00	// Status flags in EFLAGS are up-to-date
#define STM_LOG		0x01	// Flags reflect logical result in starg1
#define STM_ADD		0x02	// starg1 is result of adding to starg2
#define STM_ADDC	0x03	// "" w/ carry-in
#define STM_SUB		0x04	// starg1 is result of subtracting from starg2
#define STM_SUBB	0x05	// "" w/ borrow-in
#define STM_MUL		0x06	// Flags reflect starg1 * starg2 (unsigned)
#define STM_IMUL	0x07	// Flags reflect starg1 * starg2 (signed)
#define STM_SHIFT	0x10	// 0x10-0x1f: shift/rotate insns
#define STM_ROL		0x10	// ROL
#define STM_ROR		0x11	// ROR
#define STM_RCL		0x12	// RCL with no carry in
#define STM_RCR		0x13	// RCR with no carry in
#define STM_SHL		0x14	// SHL/SAL
#define STM_SHR		0x15	// SHR
#define STM_SAR		0x17	// SAR
#define STM_RCLC	0x1a	// RCL with a carry in
#define STM_RCRC	0x1b	// RCR with a carry in

#define STM_8		0x40	// 8-bit operand size
#define STM_16		0x80	// 16-bit operand size
#define STM_32		0xc0	// 32-bit operand size


#endif	// ENV_EMU_H
