// PowerPC instruction set definitions
#ifndef PPC_PPC_H
#define PPC_PPC_H

// I-form opcodes
#define OPI_B		((18<<26) | 0)	// Branch
#define OPI_BL		((18<<26) | 1)	// Branch & Link
#define OPI_BA		((18<<26) | 2)	// Branch Absolute
#define OPI_BLA		((18<<26) | 3)	// Branch & Link Absolute

// B-form opcodes
#define OPB_BC		((16<<26) | 0)	// Branch Conditional
#define OPB_BCL		((16<<26) | 1)	// Branch Conditional & Link
#define OPB_BCA		((16<<26) | 2)	// Branch Cond Absolute
#define OPB_BCLA	((16<<26) | 3)	// Branch Cond & Link Absolute

// XL-form opcodes
#define OPXL_BCLR	((19<<26) | (16<<1) | 0) // Branch Cond to LR
#define OPXL_BCLRL	((19<<26) | (16<<1) | 1) // Branch Cond to LR & Link
#define OPXL_BCCTR	((19<<26) | (528<<1) | 0) // Branch Cond to CTR
#define OPXL_BCCTRL	((19<<26) | (528<<1) | 1) // Branch Cond to CTR & Link

// D-form opcodes
#define OPD_LBZ		((34<<26))	// Load Byte and Zero
#define OPD_LBZU	((35<<26))	// Load Byte and Zero with Update
#define OPD_LHZ		((40<<26))	// Load Halfword and Zero
#define OPD_LHZU	((41<<26))	// Load Halfword and Zero with Update
#define OPD_LHA		((42<<26))	// Load Halfword Algebraic
#define OPD_LHAU	((43<<26))	// Load Halfword Algebraic with Update
#define OPD_LWZ		((32<<26))	// Load Word and Zero
#define OPD_LWZU	((33<<26))	// Load Word and Zero with Update
#define OPD_STB		((38<<26))	// Store Byte
#define OPD_STBU	((39<<26))	// Store Byte with Update
#define OPD_STH		((44<<26))	// Store Halfword
#define OPD_STHU	((45<<26))	// Store Halfword with Update
#define OPD_STW		((36<<26))	// Store Word
#define OPD_STWU	((37<<26))	// Store Word with Update
#define OPD_ADDI	((14<<26))	// Add Immediate
#define OPD_ADDIS	((15<<26))	// Add Immediate Shifted
#define OPD_ADDIC	((12<<26))	// Add Immediate Carrying
#define OPD_ADDIC_	((13<<26))	// Add Immediate Carrying and Record
#define OPD_SUBFIC	((8<<26))	// Subtract From Immediate Carrying

// DS-form opcodes
#define OPDS_LWA	((58<<26) | 2)	// Load Word Algebraic

// X-form opcodes
#define OPX_LBZX	((31<<26) | (87<<1))	// Load Byte and Zero Indexed
#define OPX_LBZUX	((31<<26) | (119<<1))	// Load Byte and Zero w/ Update
#define OPX_LHZX	((31<<26) | (279<<1))	// Load Halfword & Zero
#define OPX_LHZUX	((31<<26) | (311<<1))	// Load Halfword & Zero w/ Upd
#define OPX_LHAX	((31<<26) | (343<<1))	// Load Halfword Alg
#define OPX_LHAUX	((31<<26) | (375<<1))	// Load Halfword Alg w/ Upd
#define OPX_LHBRX	((31<<26) | (790<<1))	// Load Halfword Byte-Reverse
#define OPX_LWZX	((31<<26) | (23<<1))	// Load Word & Zero
#define OPX_LWZUX	((31<<26) | (55<<1))	// Load Word & Zero w/ Upd
#define OPX_LWAX	((31<<26) | (341<<1))	// Load Word Alg
#define OPX_LWAUX	((31<<26) | (373<<1))	// Load Word Alg w/ Upd
#define OPX_LWBRX	((31<<26) | (534<<1))	// Load Word Byte-Reverse
#define OPD_STBX	((31<<26) | (215<<1))	// Store Byte Indexed
#define OPD_STBUX	((31<<26) | (247<<1))	// Store Byte with Update
#define OPD_STHX	((31<<26) | (407<<1))	// Store Halfword Indexed
#define OPD_STHUX	((31<<26) | (439<<1))	// Store Halfword with Update
#define OPX_STHBRX	((31<<26) | (918<<1))	// Store Halfword Byte-Reverse
#define OPD_STWX	((31<<26) | (151<<1))	// Store Word Indexed
#define OPD_STWUX	((31<<26) | (183<<1))	// Store Word with Update
#define OPX_STWBRX	((31<<26) | (662<<1))	// Store Word Byte-Reverse

// XO-form opcodes
#define OPXO_ADD	((31<<26) | (0<<10) | (266<1) | 0)
#define OPXO_ADD_	((31<<26) | (0<<10) | (266<1) | 1)
#define OPXO_ADDO	((31<<26) | (1<<10) | (266<1) | 0)
#define OPXO_ADDO_	((31<<26) | (1<<10) | (266<1) | 1)
#define OPXO_SUBF	((31<<26) | (0<<10) | (40<1) | 0)
#define OPXO_SUBF_	((31<<26) | (0<<10) | (40<1) | 1)
#define OPXO_SUBFO	((31<<26) | (1<<10) | (40<1) | 0)
#define OPXO_SUBFO_	((31<<26) | (1<<10) | (40<1) | 1)
#define OPXO_ADDC	((31<<26) | (0<<10) | (10<1) | 0)
#define OPXO_ADDC_	((31<<26) | (0<<10) | (10<1) | 1)
#define OPXO_ADDCO	((31<<26) | (1<<10) | (10<1) | 0)
#define OPXO_ADDCO_	((31<<26) | (1<<10) | (10<1) | 1)
#define OPXO_SUBFC	((31<<26) | (0<<10) | (8<1) | 0)
#define OPXO_SUBFC_	((31<<26) | (0<<10) | (8<1) | 1)
#define OPXO_SUBFCO	((31<<26) | (1<<10) | (8<1) | 0)
#define OPXO_SUBFCO_	((31<<26) | (1<<10) | (8<1) | 1)
#define OPXO_ADDE	((31<<26) | (0<<10) | (138<1) | 0)
#define OPXO_ADDE_	((31<<26) | (0<<10) | (138<1) | 1)
#define OPXO_ADDEO	((31<<26) | (1<<10) | (138<1) | 0)
#define OPXO_ADDEO_	((31<<26) | (1<<10) | (138<1) | 1)
#define OPXO_SUBFE	((31<<26) | (0<<10) | (136<1) | 0)
#define OPXO_SUBFE_	((31<<26) | (0<<10) | (136<1) | 1)
#define OPXO_SUBFEO	((31<<26) | (1<<10) | (136<1) | 0)
#define OPXO_SUBFEO_	((31<<26) | (1<<10) | (136<1) | 1)
#define OPXO_ADDME	((31<<26) | (0<<10) | (234<1) | 0)
#define OPXO_ADDME_	((31<<26) | (0<<10) | (234<1) | 1)
#define OPXO_ADDMEO	((31<<26) | (1<<10) | (234<1) | 0)
#define OPXO_ADDMEO_	((31<<26) | (1<<10) | (234<1) | 1)
#define OPXO_SUBFME	((31<<26) | (0<<10) | (232<1) | 0)
#define OPXO_SUBFME_	((31<<26) | (0<<10) | (232<1) | 1)
#define OPXO_SUBFMEO	((31<<26) | (1<<10) | (232<1) | 0)
#define OPXO_SUBFMEO_	((31<<26) | (1<<10) | (232<1) | 1)
#define OPXO_ADDZE	((31<<26) | (0<<10) | (202<1) | 0)
#define OPXO_ADDZE_	((31<<26) | (0<<10) | (202<1) | 1)
#define OPXO_ADDZEO	((31<<26) | (1<<10) | (202<1) | 0)
#define OPXO_ADDZEO_	((31<<26) | (1<<10) | (202<1) | 1)
#define OPXO_SUBFZE	((31<<26) | (0<<10) | (200<1) | 0)
#define OPXO_SUBFZE_	((31<<26) | (0<<10) | (200<1) | 1)
#define OPXO_SUBFZEO	((31<<26) | (1<<10) | (200<1) | 0)
#define OPXO_SUBFZEO_	((31<<26) | (1<<10) | (200<1) | 1)
#define OPXO_NEG	((31<<26) | (0<<10) | (104<1) | 0)
#define OPXO_NEG_	((31<<26) | (0<<10) | (104<1) | 1)
#define OPXO_NEGO	((31<<26) | (1<<10) | (104<1) | 0)
#define OPXO_NEGO_	((31<<26) | (1<<10) | (104<1) | 1)

#endif	// PPC_PPC_H
