
#include <stdio.h>
#include <stdarg.h>
#include <fcntl.h>
#include <assert.h>
#include <sys/mman.h>

static inline long rdtsc()
{
	unsigned a, d;
	asm volatile("rdtsc" : "=d" (d), "=a" (a));
	return ((long)d << 32) | a;
}

#define REPS 100000000

int x;

#define PAGES	65500
#define TABVA	0x40000000

main()
{
	unsigned a, b, c, d;
	long t1, t2;
	int i;
	int fd;
	int rc;
	void *p;

#if 0
	asm volatile("cpuid"
		: "=a" (a), "=b" (b), "=c" (c), "=d" (d)
		: "a" (1));

	printf("cpuid %08x %08x %08x %08x\n", a, b, c, d);

	t1 = rdtsc();
	for (i = 0; i < REPS; i++)
		asm volatile("");
	t2 = rdtsc();
	printf("spin: %f cycles\n", (float)(t2-t1) / REPS);

	t1 = rdtsc();
	for (i = 0; i < REPS; i++)
		rdtsc();
	t2 = rdtsc();
	printf("rdtsc: %f cycles\n", (float)(t2-t1) / REPS);

	t1 = rdtsc();
	for (i = 0; i < REPS; i++)
		asm volatile("movl x,%%eax" : : : "rax");
	t2 = rdtsc();
	printf("load: %f cycles\n", (float)(t2-t1) / REPS);

	t1 = rdtsc();
	for (i = 0; i < REPS; i++)
		asm volatile("movl %eax,x");
	t2 = rdtsc();
	printf("store: %f cycles\n", (float)(t2-t1) / REPS);
#endif

	// Map the same page many times in our address space
	unlink("/tmp/onepage");
	fd = open("/tmp/onepage", O_RDWR | O_CREAT, 0666);
	if (fd < 0) {
		perror("open");
		exit(1);
	}
	rc = ftruncate(fd, 4096);
	assert(rc == 0);
	for (i = 0; i < PAGES; i++) {
		void *va = (void*)TABVA + i*4096;
		p = mmap(va, 4096, PROT_READ|PROT_WRITE,
			 MAP_FIXED | MAP_SHARED, fd, 0);
		assert(p == va);
	}

	// Warm the array
	for (i = 0; i < PAGES; i++)
		*((char*)TABVA + i*4096) = i;

	// Store test
	sleep(1);
	*(char*)TABVA = 0;
	t1 = rdtsc();
	for (i = 0; i < PAGES; i++)
		*((char*)TABVA + i*4096) = i;
	t2 = rdtsc();
	printf("tlbmiss store: %f cycles\n", (float)(t2-t1) / PAGES);

	// Load test
	sleep(1);
	*(char*)TABVA = 0;
	t1 = rdtsc();
	for (i = 0; i < PAGES; i++)
		asm volatile("" : : "a" (*((char*)TABVA + i*4096)));
	t2 = rdtsc();
	printf("tlbmiss load: %f cycles\n", (float)(t2-t1) / PAGES);

	// Store test
	sleep(1);
	*(char*)TABVA = 0;
	t1 = rdtsc();
	for (i = 0; i < PAGES; i++)
		*((char*)TABVA + i*4096) = i;
	t2 = rdtsc();
	printf("tlbmiss store: %f cycles\n", (float)(t2-t1) / PAGES);

	// Load test
	sleep(1);
	*(char*)TABVA = 0;
	t1 = rdtsc();
	for (i = 0; i < PAGES; i++)
		asm volatile("" : : "a" (*((char*)TABVA + i*4096)));
	t2 = rdtsc();
	printf("tlbmiss load: %f cycles\n", (float)(t2-t1) / PAGES);

	// Store test
	sleep(1);
	*(char*)TABVA = 0;
	t1 = rdtsc();
	for (i = 0; i < PAGES; i++)
		*((char*)TABVA + i*4096) = i;
	t2 = rdtsc();
	printf("tlbmiss store: %f cycles\n", (float)(t2-t1) / PAGES);



}
